require "scribble"
require "ostruct"
require "cgi"
require "schoolkeep/scribble/methods/app"
require "schoolkeep/scribble/methods/collection_each"
require "schoolkeep/scribble/methods/display_search_form"
require "schoolkeep/scribble/methods/filter_parameter_value"
require "schoolkeep/scribble/methods/hide_search_box"
require "schoolkeep/scribble/methods/l"
require "schoolkeep/scribble/methods/limit"
require "schoolkeep/scribble/methods/no_filter_selected_class"
require "schoolkeep/scribble/methods/pluralize"
require "schoolkeep/scribble/methods/query_parameter_value"
require "schoolkeep/scribble/methods/script"
require "schoolkeep/scribble/methods/t"

Scribble::Registry.for Enumerable do
  to_boolean { any? }

  method :any?

  # Logical operators
  method :or,  Object, cast: 'to_boolean'
  method :and, Object, cast: 'to_boolean'

  # Equality
  method :equals,  Enumerable, as: '=='
  method :differs, Enumerable, as: '!='
  method :equals,  Object, returns: false
  method :differs, Object, returns: true
end

Scribble::Registry.for ActiveRecord::Associations::CollectionProxy do
  to_boolean { any? }

  # Logical operators
  method :or,  Object, cast: 'to_boolean'
  method :and, Object, cast: 'to_boolean'

  # Equality
  method :equals,  ActiveRecord::Associations::CollectionProxy, as: '=='
  method :differs, ActiveRecord::Associations::CollectionProxy, as: '!='
  method :equals,  Object, returns: false
  method :differs, Object, returns: true
end

Scribble::Registry.for Draper::CollectionDecorator do
  method :count
  method :any?
end

module Schoolkeep
  module Scribble
    class << self
      attr_accessor :stub

      def register_models
        if stub
          models.each do |class_name, args|
            begin
              constantize(class_name)
            rescue NameError
              Object.const_set(class_name, Class.new(OpenStruct))
            end
          end
        end

        models.each do |class_name, args|
          register_model(class_name, *args)
        end
      end

      private

      def register_model(class_name, boolean, block)
        ::Scribble::Registry.instance.class_names.delete_if do |klass, name|
          name == class_name
        end

        ::Scribble::Registry.instance.methods.reject! do |method|
          method.receiver_class.name == class_name
        end

        klass = constantize(class_name)

        ::Scribble::Registry.for klass do
          name class_name
          to_boolean { boolean }

          # Logical operators
          method :or,  Object, cast: 'to_boolean'
          method :and, Object, cast: 'to_boolean'

          # Equality
          method :equals,  klass, as: '=='
          method :differs, klass, as: '!='
          method :equals,  Object, returns: false
          method :differs, Object, returns: true
        end

        ::Scribble::Registry.for klass, &block
      end

      def models
        @models ||= {}
      end

      def register(class_name, boolean: true, &block)
        models[class_name.to_s] = [boolean, block]
      end

      def constantize(camel_cased_word)
        names = camel_cased_word.split('::')

        # Trigger a builtin NameError exception including the ill-formed constant in the message.
        Object.const_get(camel_cased_word) if names.empty?

        # Remove the first blank element in case of '::ClassName' notation.
        names.shift if names.size > 1 && names.first.empty?

        names.inject(Object) do |constant, name|
          if constant == Object
            constant.const_get(name)
          else
            candidate = constant.const_get(name)
            next candidate if constant.const_defined?(name, false)
            next candidate unless Object.const_defined?(name)

            # Go down the ancestors to check it it's owned
            # directly before we reach Object or the end of ancestors.
            constant = constant.ancestors.inject do |const, ancestor|
              break const    if ancestor == Object
              break ancestor if ancestor.const_defined?(name, false)
              const
            end

            # owner is in Object, so raise
            constant.const_get(name, false)
          end
        end
      end
    end

    register :Category do
      method :active?
      method :link
      method :name
    end

    register :CustomPage do
      to_string  { "custom_page: #{page_title}" }

      # Attributes
      method :page_title
      method :headline
      method :subheadline
      method :image_url
      method :content
    end

    register :School do
      to_string  { "school: #{name}" }

      # Attributes
      method :disable_social_share?
      method :filterable_categories # DEPRECATED
      method :filtering_enabled?
      method :has_custom_styles?
      method :locales
      method :logo_navigation_url
      method :logo_url
      method :my_content_label
      method :my_content_path # DEPRECATED
      method :name
      method :public
      method :search_enabled?
      method :show_powered_by_school_keep?
      method :sso_enabled?
    end

    register :WebsiteNavigation do
      to_string  { "website navigation: #{name}" }

      method :name
      method :path
      method :external?
    end

    register :WebsiteFooter do
      to_string  { "website footer" }

      method :show_navigation_links?
      method :show_social_media_links?
      method :show_customer_service_email?
      method :school_customer_service_email
      method :social_media_links
    end

    register :SocialMediaLink do
      to_string  { "social media link: #{name}" }

      method :link
      method :name
    end

    register :Course do
      to_string  { "course: #{name}" }

      method :course_category_names
      method :full_description
      method :has_outline?
      method :name
      method :payment_required?
      method :published_sections
      method :ribbon
      method :short_description
      method :show_events?
    end

    register :VirtualEventSessionSummary do
      to_string  { "virtual event session summary" }

      method :virtual_event_name
      method :provider_logo_url
      method :virtual_event_start_url
      method :starts_at
      method :ends_at
      method :join_link
    end

    register "Search::CatalogResult" do
      method :count
    end

    register "SchoolWebsite::MyContentDecorator" do
      to_string  { "my content decorator" }

      method :courses
      method :headline
      method :subheadline
    end

    register :Section do
      to_string { "section: #{name}" }

      method :name
      method :activities
    end

    register "SchoolWebsite::ActivityDecorator" do
      method :completed?
      method :list_class
      method :milestone?
      method :path
      method :title
    end

    register "SchoolWebsite::CourseDecorator" do
      to_string  { "course decorator: #{name}" }

      method :about_label
      method :buy_label
      method :course_attempts_path
      method :cover_path
      method :details_path
      method :details_url
      method :enrollment_url
      method :featured_image_url
      method :formatted_price
      method :forum_path
      method :go_to_label
      method :has_forum?
      method :instructor_names
      method :instructors
      method :instructors_title
      method :virtual_events_title
      method :learner_can_retake?
      method :list_image_url
      method :new_order_path
      method :outline_label
      method :outline_path
      method :partnership_names
      method :partnerships
      method :progress
      method :progress_bar
      method :progress_text, Person
      method :progress_title
      method :promo_video_embed
      method :register_for_events
      method :retake_course
      method :social_links
      method :start_or_continue
      method :superscript
      method :upcoming_event_sessions
      method :view_label
      method :virtual_events_path
      method :discover_events_path
    end

    register "SchoolWebsite::CourseCatalogDecorator" do
      to_string  { "course decorator: #{name}" }

      method :headline
      method :headline_empty_collection
      method :subheadline
      method :subheadline_empty_collection
    end

    register "SchoolWebsite::UpcomingRegisteredEventsDecorator" do
      method :sessions
      method :additional_sessions?
    end

    register "SchoolWebsite::TrainingSessionsDecorator" do
      method :head
      method :tail
    end

    register "SchoolWebsite::TrainingSessionDecorator" do
      method :status
      method :title
      method :starts_at
      method :waiting?
      method :pending?
      method :join_url
      method :logmein?
      method :in_person?
      method :room_name
      method :location_name
    end

    register "Partnerships::Educator" do
      to_string { "partnership: #{name}" }

      method :image_url
      method :name
      method :title
      method :bio
    end

    register "LearnerSyllabuses::SectionsCollection" do
    end

    register "LearnerSyllabuses::ActivitiesCollection" do
    end

    register "LearnerSyllabuses::Section" do
      method :activities
      method :name
      method :published_at
      method :will_be_published?
    end

    register "LearnerSyllabuses::Activity" do
      method :completed?
      method :list_class
      method :locked?
      method :milestone?
      method :path
      method :title
    end

    register "LearnerSyllabuses::Syllabus" do
      method :sections
    end

    register :Learner do
      to_string  { "learner: #{name}" }

      # Attributes
      method :access_course?, Course
      method :enrolled_in_course?, Course
      method :filterable_categories
      method :my_content_label
      method :name
      method :email
      method :sso_uid
      method :id
      method :upcoming_registered_events
    end

    register :Person do
      to_string  { "learner: #{name}" }

      # Attributes
      method :access_course?, Course
      method :enrolled_in_course?, Course
      method :filterable_categories
      method :my_content_label
      method :name
      method :email
      method :sso_uid
      method :id
      method :upcoming_registered_events
    end

    register :GuestLearner, boolean: false do
      to_string  { "guest learner" }

      method :access_course?, Course
      method :enrolled_in_course?, Course
      method :filterable_categories
      method :my_content_label
      method :name
      method :email
      method :sso_uid
      method :id
      method :upcoming_registered_events
    end

    register :GuestPerson, boolean: false do
      to_string  { "guest learner" }

      method :access_course?, Course
      method :enrolled_in_course?, Course
      method :filterable_categories
      method :my_content_label
      method :name
      method :email
      method :sso_uid
      method :id
      method :upcoming_registered_events
    end

    register :Homepage do
      to_string  { "homepage" }

      method :headline
      method :subheadline
      method :artwork_url
      method :featured_courses_headline
      method :featured_courses_subheadline
      method :published_featured_courses
    end

    register "Schools::Routes" do
      to_string { "school routes" }

      method :my_courses_path
      method :my_content_path
      method :my_profile_path
      method :log_out_path
      method :log_in_path
      method :sign_up_path

      method :color_scheme_path
      method :custom_styles_path
      method :schoolkeep_v3_styles_url
      method :school_website_search_path
    end

    register "ColorScheme" do
      to_string { "Color Scheme" }

      method :background_color
      method :background_font_color
      method :button_color
      method :button_font_color
      method :button_hover_color
      method :banner_font_color
      method :header_color
      method :header_font_color
      method :header_font_hover_color
    end

    register "BaseApp" do
      to_string { "Base App" }

      method :active?
      method :inactive?
    end

    register :Locale do
      to_string { "Locale" }

      method :code
      method :name
    end
  end
end
