require "webrick"
require "schoolkeep"
require "schoolkeep/template"
require "schoolkeep/fixture"
require "schoolkeep/views/layout"
require "schoolkeep/views/color_scheme"

module Schoolkeep
  class Server
    attr_reader :templates_path, :port, :quiet

    def initialize(dir: ".", port: 4000, quiet: false, asset_host: ASSET_HOST)
      @templates_path = File.join(dir, "templates")
      Template.path = templates_path
      fixture_path = File.join(dir, "config/fixtures.yml")
      unless File.exist?(fixture_path)
        fixture_path = File.join(dir,"fixtures.yml")
      end
      Fixture.default_path =  fixture_path
      Views::Layout.asset_host = asset_host
      @port = port
      @quiet = quiet

      webrick.mount_proc "/color_scheme.css" do |req, res|
        res["content-type"] = "text/css"
        res.body = Views::ColorScheme.new("color_scheme").render
      end

      webrick.mount_proc "/learning_color_scheme.css" do |req, res|
        res["content-type"] = "text/css"
        res.body = Views::ColorScheme.new("learning_color_scheme").render
      end

      webrick.mount_proc "/catalog/" do |req, res|
        id = req.path.split("/")[2]
        if id
          template = Template.new("course_details.html.sktl")

          res["content-type"] = template.type
          res.body = Views::Layout.new(template).render(variables: { "course" => id.to_sym })
        else
          template = Template.new("course_index.html.sktl")

          res["content-type"] = template.type
          res.body = Views::Layout.new(template).render
        end
      end

      webrick.mount_proc "/syllabus/" do |req, res|
        id = req.path.split("/")[2]
        template = Template.new("course_cover.html.sktl")

        res["content-type"] = template.type
        res.body = Views::Layout.new(template).render(variables: { "course" => id.to_sym })
      end

      webrick.mount_proc "/outline/" do |req, res|
        id = req.path.split("/")[2]

        template = Template.new("course_cover.html.sktl")
        res["content-type"] = template.type
        res.body = Views::Layout.new(template).render(variables: {
          "course" => id.to_sym
        })
      end
    end

    def start
      webrick.start
    end

    def shutdown
      webrick.shutdown
    end

    private

    def webrick
      @webrick ||= WEBrick::HTTPServer.new config
    end

    def config
      {
        Port: port,
        DocumentRoot: templates_path,
        DocumentRootOptions: {
          FancyIndexing: true,
          HandlerTable: { "sktl" => SKTLHandler },
          NondisclosureName: ["_*"]
        }
      }.tap do |c|
        if quiet
          c.merge!(
            :AccessLog => [],
            :Logger => WEBrick::Log::new(File::NULL, 7)
          )
        end
      end
    end

    class SKTLHandler < WEBrick::HTTPServlet::AbstractServlet
      def initialize(server, name)
        super(server, name)
        @script_filename = name
      end

      def do_GET(req, res)
        begin
          name = File.basename(@script_filename)
          template = Template.new(name)
          res["content-type"] = template.type

          if template.type == Template::HTML_MIME
            res.body = Views::Layout.new(template).render
          else
            res.body = template.render
          end
        rescue StandardError
          raise
        rescue Exception => ex
          @logger.error(ex)
          raise HTTPStatus::InternalServerError, ex.message
        end
      end

      ##
      # Handles POST requests

      alias do_POST do_GET
    end
  end
end
